@@ -45,6 +45,9 @@ def createSignal(G, nodes=None):
     if nodes is None:
         s = np.identity(G.N)
     else:
+        if not hasattr(nodes, '__iter__'):
+            nodes = [nodes]
+
         s = np.zeros((G.N, len(nodes)))
         for i, n in enumerate(nodes):
             s[n][i] = 1.0
@@ -52,7 +55,7 @@ def createSignal(G, nodes=None):
 
 def _tig(g, s, **kwargs):
     """
-    Returns a tensor of (nodes, coeffs, filters)
+    Returns a tensor of (coeffs, nodes, filters)
     """
     tig = g.filter(s[..., np.newaxis], **kwargs)
     if s.shape[1] == 1: # single node
@@ -139,11 +142,17 @@ def graphWave(G, g=None, ts=None, nodes=None, ecf_method=None, **kwargs):
     G.gw = gw
     return gw
 
-def spectrogram(G, g=None, M=100, nodes=None, **kwargs):
+def spectrogram(G, g=None, M=100, shifts=None, nodes=None, **kwargs):
     if g is None:
         g = GaussianFilter(G, M)
     assert g.G == G
-    g = ShiftedFilter(g, M)
+
+    if shifts is None:
+        Nf = M
+    else:
+        Nf = len(shifts)
+    g = ShiftedFilter(g, Nf, shifts)
+
     norm_sqr_func = lambda tig: np.linalg.norm(tig, axis=0, ord=2)**2
     spectr = nodeEmbedding(g, norm_sqr_func, nodes, **kwargs)
     G.spectr = spectr
@@ -382,12 +391,12 @@ class LGraphFourier(gsp.graphs.fourier.GraphFourier):
         self._e[np.isclose(self._e, 0)] = 0
         self._lmin = self._e[np.where(self._e>0)][0]
 
-        e_max = np.max(self._e)
-        assert e_max == self._e[-1]
         e_bound = self._get_upper_bound()
         self._e[np.isclose(self._e, e_bound)] = e_bound
+        e_max = np.max(self._e)
+        assert e_max == self._e[-1]
         assert e_max <= e_bound, "Maximum eigenvalue was {} but upper bound is {}".format(e_max, e_bound)
-        self._lmax = self._e[-1]
+        self._lmax = e_max
 
     def _get_upper_bound(self):
         max_edge_degree = np.max(self.iw)
@@ -501,9 +510,13 @@ class GWHeat(gsp.filters.Filter):
     """
     Heat kernel used by GraphWave.
     """
-    def __init__(self, G, Nf=2, approximate=False, gamma=0.95, eta=0.85, maxiter=2000):
-        def kernel(x, s):
-            return np.exp(-x * s)
+    def __init__(self, G, Nf=2, scale=False, normalize=False, gamma=0.95, eta=0.85, approximate=False, maxiter=2000):
+        if scale:
+            def kernel(x, s):
+                return np.exp(-x * s / G.lmax)
+        else:
+            def kernel(x, s):
+                return np.exp(-x * s)
 
         if hasattr(G, '_lmin'):
             lmin = G._lmin
@@ -523,7 +536,13 @@ class GWHeat(gsp.filters.Filter):
         assert s_min < s_max
         # log scale
         scales = np.exp(np.linspace(np.log(s_min), np.log(s_max), Nf))
-        kernels = [lambda x, s=s: kernel(x, s) for s in scales]
+        kernels = []
+        for s in scales:
+            if normalize:
+                norm = np.linalg.norm(kernel(G.e, s))
+                kernels.append(lambda x, s=s, norm=norm: kernel(x, s)/norm)
+            else:
+                kernels.append(lambda x, s=s: kernel(x, s))
 
         super().__init__(G, kernels)
 
@@ -541,9 +560,15 @@ class ShiftedFilter(gsp.filters.Filter):
     """
     Filter used by spectrogram.
     """
-    def __init__(self, g, Nf=100):
+    def __init__(self, g, Nf=100, shifts=None):
         G = g.G
-        shifts = np.linspace(0, G.lmax, Nf)
+
+        if shifts is None:
+            shifts = np.linspace(0, G.lmax, Nf)
+
+        if len(shifts) != Nf:
+            raise ValueError('len(shifts) should be Nf.')
+
         kernels = [lambda x, kernel=kernel, s=s: kernel(x - s) for s in shifts for kernel in g._kernels]
 
         super().__init__(G, kernels)
